%
% Copyright (c) 2015, Yarpiz (www.yarpiz.com)
% All rights reserved. Please read the "license.txt" for license terms.
%
% Project Code: YPML110
% Project Title: Implementation of DBSCAN Clustering in MATLAB
% Publisher: Yarpiz (www.yarpiz.com)
% 
% Developer: S. Mostapha Kalami Heris (Member of Yarpiz Team)
% 
% Contact Info: sm.kalami@gmail.com, info@yarpiz.com
%
%==========================================================================
% The code is modifiedby E.Tijskens (engelbert.tijskens@uantwerpen.be) with
% an extra argument row_index. The distance between points with the same
% row_index is set artificially to a very large number to avoid they would
% end up in the same cluster.
%==========================================================================

function [IDX, isnoise]=DBSCANm(X,epsilon,MinPts,row_index)

    C=0;
    
    n=size(X,1);
    IDX=zeros(n,1);
    
    D=pdist2(X,X);
    
    % Modification by E.Tijskens:
    if nargin==4
        % Artificially set the distance between points with the same
        % row_index value to a very large number that prevents them to end
        % up in the same cluster:
        very_large = 1000*epsilon;
        for i1=2:n
            for i2=1:i1-1
                if row_index(i1)==row_index(i2)
                    D(i1,i2) = very_large;
                    D(i2,i1) = very_large;
                end
            end
        end
    end
    
    visited=false(n,1);
    isnoise=false(n,1);
    
    for i=1:n
        if ~visited(i)
            visited(i)=true;
            
            Neighbors=RegionQuery(i);
            if numel(Neighbors)<MinPts
                % X(i,:) is NOISE
                isnoise(i)=true;
            else
                C=C+1;
                ExpandCluster(i,Neighbors,C);
            end
            
        end
    
    end
    
    function ExpandCluster(i,Neighbors,C)
        IDX(i)=C;
        
        k = 1;
        while true
            j = Neighbors(k);
            
            if ~visited(j)
                visited(j)=true;
                Neighbors2=RegionQuery(j);
                if numel(Neighbors2)>=MinPts
                    Neighbors=[Neighbors Neighbors2];   %#ok
                end
            end
            if IDX(j)==0
                IDX(j)=C;
            end
            
            k = k + 1;
            if k > numel(Neighbors)
                break;
            end
        end
    end
    
    function Neighbors=RegionQuery(i)
        Neighbors=find(D(i,:)<=epsilon);
    end

end


