function assert_valid( object, typecheck, varname, caller )
    % [function] This function tests whether *object* is an object that
    % satisfies the type description _typecheck_ and raises an error if
    % not.
    %
    % :param object: object to be tested.
    % :param typecheck: class name, or function handle, or  cell array of
    %   class names and/or function handles. The function handles must
    %   belong to functions returning true (accept) or false (reject).
    % :param [varname]: string with the name of the variable containing the
    %   value, or a FunctionArgument object. Default is empty string. Used
    %   by the error message. 
    % :param [caller]: string identifying who requested this assertion.
    %   Default is empty string. Used by the error message. If _varname_ is
    %   a FunctionArgument object its signature is used for the caller.
    % :return: logical.
    %
    % Note that if *classes* is empty the test succeeds.
    %
    
    if ~validate_value(object,typecheck)
        switch nargin
            case 2
                varname = '';
                caller  = '';
            case 3
                caller  = '';
            case 4

            otherwise
                Sparsimatics.Error(''...
                  ,'assert_valid( object, typecheck [,varname[,caller]] )'...\
                  ,'Expecting from 2 to 4 input arguments, got %d', nargin ...
                  );
        end
        if ~isempty(varname)
            varname = [' for ''' varname ''''];
        end
        
        if ischar(typecheck)
            typestr = typecheck;
        elseif iscell(typecheck)
            typestr = '';
            for i=1:length(typecheck)
                if ischar(typecheck{i})
                    typestr = [typestr '|' typecheck{i}]; %#ok<AGROW>
                elseif iscell(typecheck{i})
                    s = typecheck{i}{1};
                    for j=2:numel(typecheck{i})
                        s = [s '&' typecheck{i}{j}]; %#ok<AGROW>
                    end
                    typestr = [typestr '|' s]; %#ok<AGROW>
                else
                    assert(false,'Bad typecheck description!')
                end
            end            
        else
            assert(false,'Bad typecheck description!')
        end
        
        Sparsimatics.Error( caller ...
          , 'Expecting object of type ''%s'', got ''%s''.'...
          ,  typestr, obj2str(object) ...
          );
    end
end

