function draw_circle( center, radius, color, style, plot_center )
    % [function] draw a circle in the current figure (2D).
    % 
    % :param center: the coordinates of the center of the circle, either an
    %   array of two real values, or a complex number. default = [0 0].
    % :param radius: the radius of the circle
    % :param color: the color of the circle
    % :param style: the line style of the circle
    % :param plot_center: true|false, draw the center point, or not.
    %
    
    if nargin>5
        Sparsimatics.Error('function draw_circle( center, radius, color, style )' ...
          , 'Can accept from 0 to 5 arguments, got %d.' ...
          , nargin ...
          );
    end
    if nargin<=4
        plot_center = false;
    end
    if nargin<=3
        style = '-';
    end
    if nargin<=2
        color = [0 0 0];
    end
    if nargin<=1
        radius = 1;
    end
    if nargin==0
        zcenter = 0;
    else
        switch numel(center)
            case 1
                zcenter = center;
            case 2 
                zcenter = center(1)+1i*center(2);
            otherwise
                Sparsimatics.Error('function draw_circle( center, radius, color, style )' ...
                  , 'Center must be double array of length 2, or a complex number.' ...
                  , nargin ...
                  );
        end         
    end
    
    hold on

    current_legend_handle = findobj(gcf,'Tag','legend');
    if ~isempty(current_legend_handle)
        current_legend_handle.AutoUpdate = 'off';
    end
    
    persistent z
    if isempty(z)
        n=1000;
        z = exp((2*pi/n)*1i*(0:n));
    end
    
    plot( radius*z + zcenter, style, 'Color', color );
    if plot_center
        plot(        zcenter, 'o'  , 'Color', color );
    end
    hold off 
    if ~isempty(current_legend_handle)
        current_legend_handle.AutoUpdate = 'off';
    end
end

