function rv = row_vector( v )
    % [function] convert vector in row vector.
    % 
    % :param double v: vector
    % :returns: if v is a column vector or a matrix, it is converted to a
    %   row vector and returned. Otherwise, an empty array is returned.
    %
    % Typical use::
    %
    %	r = row_vector(a);
    %	if ~isempty(r)
    %	    a = r;
    %	end
    %
    % So if r is already a row vector it is not reassigned.

    shape = size(v);
    if shape(1)==1 && numel(v)==length(v)
        rv = [];
        return
    end
    rv = reshape(v,1,numel(v));
end

