function vdm = vandermonde_jk( b, nrows, j, k )
    % [function] This function constructs the vandermonde matrix of a row
    % vector *b*. The vandermonde matrix is defined as::
    %
    %   V = [ b .^ (0*k+j) ;
    %         b .^ (1*k+j) ;
    %         b .^ (2*k+j) ;
    %         ...
    %         b .^ (nrows*k+j) ];
    %
    % :param b: A row vector.
    % :param nrows: the number of rows in the vandermonde matrix *V*.
    % :param j: integer, see definition above, default=0.
    % :param k: integer, see definition above, default=1.
    % :return: the vandermonde matrix of size *(nrows,length(b))*, see below.
    %

% removed varargin
%     j=0;
%     k=1;
%     if nargin>2
%         j = varargin{1};
%         if nargin>3
%             k = varargin{2};
%             if nargin>4
%                 Sparsimatics.Error('','vandermonde( b, nrows [, k] )'...
%                   ,'Expecting 2 or 3 input arguments, got %d'...
%                   , nargin ...
%                   );
%             end
%         end
%     end
    ncols = length(b);    
    if nrows==0 || ncols==0
        vdm = [];
    elseif nrows==1
        if j==0
            vdm = ones(1,ncols);
        else
            vdm = reshape(b,[1,ncols]) .^ j;
        end
    else
        b_hat_k = reshape(b,[1,ncols]) .^ k;
        vdm = ones(nrows,ncols);
        if j>0
            vdm(1,:) = reshape(b,[1,ncols]) .^ j;
        end
        for i=2:nrows
            vdm(i,:) = vdm(i-1,:) .* b_hat_k;
        end
    end
end

